@@ -58,6 +58,7 @@
 
 
 
+static void      panel_application_dispose            (GObject                *object);
 static void      panel_application_finalize           (GObject                *object);
 static gboolean  panel_application_autosave_timer     (gpointer                user_data);
 static void      panel_application_plugin_move        (GtkWidget              *item,
@@ -186,6 +187,7 @@ panel_application_class_init (PanelApplicationClass *klass)
   GObjectClass *gobject_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
+  gobject_class->dispose = panel_application_dispose;
   gobject_class->finalize = panel_application_finalize;
 }
 
@@ -231,6 +233,25 @@ panel_application_init (PanelApplication *application)
 
 
 
+static void
+panel_application_dispose (GObject *object)
+{
+  PanelApplication *application = PANEL_APPLICATION (object);
+
+  /* save plugins: xfconf_shutdown() is called via a weak ref i.e. on dispose(),
+   * so this should be done here to avoid any use-after-free */
+  if (application->autosave_timer_id != 0)
+    {
+      g_source_remove (application->autosave_timer_id);
+      application->autosave_timer_id = 0;
+      panel_application_autosave_timer (application);
+    }
+
+  (*G_OBJECT_CLASS (panel_application_parent_class)->dispose) (object);
+}
+
+
+
 static void
 panel_application_finalize (GObject *object)
 {
@@ -238,14 +259,6 @@ panel_application_finalize (GObject *object)
 
   panel_return_if_fail (application->dialogs == NULL);
 
-  if (application->autosave_timer_id != 0)
-    {
-      g_source_remove (application->autosave_timer_id);
-
-      /* save plugins */
-      panel_application_autosave_timer (application);
-    }
-
 #ifdef GDK_WINDOWING_X11
   /* stop autostart timeout */
   if (application->wait_for_wm_timeout_id != 0)
@@ -553,9 +566,6 @@ panel_application_plugin_move_drag_end (GtkWidget        *item,
   g_signal_handlers_disconnect_by_func (G_OBJECT (item),
       G_CALLBACK (panel_application_plugin_move_drag_data_get), application);
 
-  /* release extra reference */
-  g_object_unref (item);
-
   /* unblock autohide */
   panel_application_windows_blocked (application, FALSE);
 }
@@ -575,9 +585,6 @@ panel_application_plugin_move (GtkWidget        *item,
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (item));
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
 
-  /* keep a reference to prevent any use-after-free when DnDing between different panels */
-  g_object_ref (item);
-
   /* block autohide */
   panel_application_windows_blocked (application, TRUE);
 
