@@ -30,6 +30,7 @@
 #include <libdbusmenu-gtk/dbusmenu-gtk.h>
 #endif
 
+#include <common/panel-debug.h>
 #include "sn-item.h"
 
 
@@ -143,7 +144,12 @@ if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) \
     return; \
   } \
 if (error != NULL) \
-  g_error_free (error);
+  {  \
+    panel_debug (PANEL_DEBUG_SYSTRAY, "%s: Fatal error for item '%s': (domain '%s', code %d) %s", \
+                 G_STRLOC, XFCE_IS_SN_ITEM (item) ? item->id : "", \
+                 g_quark_to_string (error->domain), error->code, error->message); \
+    g_error_free (error); \
+  }
 
 
 
@@ -151,7 +157,11 @@ if (error != NULL) \
 if (condition) \
   { \
     if (G_IS_OBJECT (item)) \
-      g_signal_emit (item, sn_item_signals[FINISH], 0); \
+      { \
+        panel_debug (PANEL_DEBUG_SYSTRAY, "%s: Finishing on error for item '%s'", \
+                     G_STRLOC, item->id); \
+        g_signal_emit (item, sn_item_signals[FINISH], 0); \
+      } \
     return; \
   }
 
@@ -484,7 +494,9 @@ sn_item_start_failed (gpointer user_data)
 {
   SnItem *item = user_data;
 
-  /* start is failed, emit the signel in next loop iteration */
+  /* start is failed, emit the signal in next loop iteration */
+  panel_debug (PANEL_DEBUG_SYSTRAY, "%s: Finishing on error for item '%s'",
+               G_STRLOC, XFCE_IS_SN_ITEM (item) ? item->id : "");
   g_signal_emit (G_OBJECT (item), sn_item_signals[FINISH], 0);
 
   return G_SOURCE_REMOVE;
@@ -565,6 +577,12 @@ sn_item_signal_received (GDBusProxy *proxy,
     }
   else if (!g_strcmp0 (signal_name, "NewStatus"))
     {
+      if (parameters == NULL || ! g_variant_check_format_string (parameters, "(s)", FALSE))
+        {
+          g_warning ("Could not parse properties for StatusNotifierItem.");
+          return;
+        }
+
       g_variant_get (parameters, "(s)", &status);
       exposed = sn_item_status_is_exposed (status);
       g_free (status);
@@ -710,8 +728,11 @@ sn_item_get_all_properties_result (GObject      *source_object,
   gboolean      update_menu = FALSE;
 
   properties = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object), res, &error);
-  free_error_and_return_if_cancelled (error);
-  finish_and_return_if_true (properties == NULL);
+  if (properties == NULL)
+    {
+      free_error_and_return_if_cancelled (error);
+      return;
+    }
 
   #define string_empty_null(s) ((s) != NULL ? (s) : "")
 
